/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("com.ibm.team.repository.web.ui.internal.LoginWidget");
dojo.require("dojo.cookie");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");

//
// Provides a generic remoting mechanism to the team server's REST services.
// @since 0.5
//

//
// Aliases
//

var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;

/**
 * Invokes the service specified by <code>serviceRequest</code>
 * asyncronously and calls the appropriate function specified by
 * <code>serviceResponseHandler</code> in the context of
 * <code>serviceResponseHandler</code>.
 *
 * If the service returns successfully, invokeService will call back to
 * the function defined by handler.getCallbackName(); if the service call
 * results in an error, invokeService will call back to the function defined
 * by handler.getErrorCallbackName().
 * 
 * @since 0.5
 */
function invokeService(serviceRequest, serviceResponseHandler) {
	var bindArgs = _getBindArgs(serviceRequest, serviceResponseHandler);
	// check for the secret sync parameter for unit testing
	if (arguments[2] === true) {
		bindArgs.sync= true;
	}
	var xhr = null;
	if (serviceRequest.getMethodType() == "POST") {
		bindArgs.postData = serviceRequest.getQueryString();
		xhr = dojo.rawXhrPost(bindArgs).ioArgs.xhr;
	}
	else {
		xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
	}
	serviceRequest._setXMLHttpRequest(xhr);
}

com.ibm.team.repository.web.transport.TeamServerClient.invokeService = invokeService;

//
// Internal consts and functions
//

var SERVICE_PREFIX   = dojo.baseUrl + "../../service/";
var X_AUTH_MSG = "X-com-ibm-team-repository-web-auth-msg";
var USERID_COOKIE = "JazzLoginCookie";

function _getBindArgs(serviceRequest, serviceResponseHandler) {
	var headers = {};
	headers["accept"] = "text/json";
	headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
	var bindArgs = {
		url: _assembleURL(serviceRequest),
		handleAs: "json",
		headers: headers,
		preventCache: dojo.isIE,
		handle: function(response, ioArgs){
			if (response instanceof Error) {
				// For work item 18631 ('Propogate server exception details to
				// REST service clients'), we can access the error response's
				// body via xhr.responseText.  Will probably have to manually
				// demarshall this data here, since I don't believe Dojo
				// provides any auto-demarshalling on error paths.

				// temporary fix for Defect 32729
				// test data (what we expect WebSphere to send back on ITEs)
				//ioArgs.xhr = {
				//	status: 500,
				//	statusText: "Internal Server Error",
				//	responseText: "Error 500: -- &#39;Save Work Item&#39; was blocked by process.\nThe work item is not assigned to a category.\n"
				//}
				var errorObj = {
					status: ioArgs.xhr.status,
					message: null,
					exception: undefined
				};
				if (ioArgs.xhr.statusText === "Internal Server Error" || ioArgs.xhr.statusText === "Bad Request") {
					var errorResponse = ioArgs.xhr.responseText || "";
					errorResponse = dojo.fromJson(errorResponse);
					if (dojo.isObject(errorResponse)) {
						errorObj.message = "["+errorResponse.errorCode + "] : " + errorResponse.errorMessage;
						errorObj.exception = errorResponse;
					}
					else {
						errorObj.message = errorResponse;
						if (errorObj.message.indexOf("-- ") > 0 && errorObj.message.length > 3) {
							errorObj.message = errorObj.message.substring(errorObj.message.indexOf("-- ") + 3);
						}
						errorObj.message = errorObj.message.replace(/&#39;/g, "\"");
					}
				} else if (ioArgs.xhr.status == 401 && ioArgs.args.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
					// Capture the scenario where the user cancels out of a Basic auth prompt and display a helpful message
					// rather than a blank "Loading..." screen
					var errMsg = messages.authRequiredRefresh;
					errorObj.message = errMsg; 					
				} else if (ioArgs.xhr.status == 403 && ioArgs.args.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
					// Capture the scenario where a user account does not belong to the groups required to access the Web UI
					// Provide a helpful error message rather than a blank "Loading..." screen.
					var errMsg = messages.noAccessWeb;
					errorObj.message = errMsg;
				} else {
					errorObj.message = ioArgs.xhr.statusText;
				}
				
				errorObj.message = dojo.string.trim(unescape(errorObj.message).replace(/\+/g, " ").replace(/^(-*)/g, ""));
					
				serviceResponseHandler.doErrorCallback(errorObj);
			} else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authrequired") {
				_loginPrompt(this, serviceResponseHandler, serviceRequest.getMethodType().toLowerCase());
			} else {
				var returnValue = _demarshallResponse(response);
				callContributedCode( function() {
					serviceResponseHandler.doCallback(returnValue);
				});
			}			
		}
	};
	return bindArgs;
}

function _loginPrompt(bindArgs, serviceResponseHandler, requestMethod, noQueue, errMessage) {
	var workbenchNode = dojo.byId("workbench");
	var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
	if (dialogWidget == null) {
		var loginWidget = new com.ibm.team.repository.web.ui.internal.LoginWidget(null, workbenchNode);
		dialogWidget = new dijit.Dialog({id:"com-ibm-team-repository-internal-LoginWidgetDialog",
			execute: dojo.hitch(this, function(dialogArgs) {
				// allow the dialogWidget to be closed
				dialogWidget._alreadyInitialized = true;
				dialogWidget.hide();
				dialogWidget.loginState = "inactive";
				if (dialogArgs.rememberUserID == "on") {
					dojo.cookie(USERID_COOKIE, dialogArgs.userid, { expires: 365 });
				} else {
					dojo.cookie(USERID_COOKIE, "");
				}
				_login(dialogArgs.userid, dialogArgs.password, bindArgs, serviceResponseHandler, requestMethod);
			})
		}, loginWidget.domNode);
		dialogWidget.loginWidget = loginWidget;
		dialogWidget.loginState = "inactive";
		loginWidget.connect(loginWidget.loginButton,"onclick",function(event) {
			dialogWidget.execute(dialogWidget.getValues());
		});
		loginWidget.connect(loginWidget.passwordNode.domNode,"onkeypress",function (event) {
			// If the user hits "Enter" in the password field, submit the data
			if (event.keyCode == dojo.keys.ENTER) {
				dialogWidget.execute(dialogWidget.getValues());
			}
		});
	}
	if (errMessage != null) {
		dialogWidget.loginWidget.showErrorMessage(errMessage);
	}
	else {
		dialogWidget.loginWidget.hideErrorMessage();
	}
	var storedUserId = dojo.cookie(USERID_COOKIE);
	if (storedUserId) {
		dialogWidget.loginWidget.setUserId(storedUserId);
		dialogWidget.loginWidget.rememberUserIdNode.setChecked(true);
		var i = 1;
	} else {
		var i = 0;
	}
	if (dialogWidget.loginState == "inactive") {
		document.title = messages.loginRequired + " - " + messages.jazzTeamServer;
		dialogWidget.show();
		setTimeout(dojo.hitch(dialogWidget, function(){
			dijit.focus(dojo.query('input', this.domNode)[i]); 
		}), 50);
		// prevent the user from closing the dialogWidget without submitting
		dialogWidget._alreadyInitialized = false;
		dialogWidget.loginState = "active";
	}
	if (dialogWidget.queuedRequests == null) {
		dialogWidget.queuedRequests = new Array();
	}
	if (noQueue == null || !noQueue) {
		dialogWidget.queuedRequests.push({
			bindArgs: bindArgs,
			requestMethod: requestMethod,
			responseHandler: serviceResponseHandler
		});
	}
}

function _login(userid, password, bindArgs, serviceResponseHandler, requestMethod) {
	this._previousRequest = bindArgs;
    var bindArgs= {
		contentType: "application/x-www-form-urlencoded; charset=utf-8",
        url: "j_security_check",
        content: {
            j_username: userid,
            j_password: password
        },
		handleAs: "json",
		headers: {
			accept: "text/json"
		},
		handle: dojo.hitch(this, function(response, ioArgs){
			var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
			dialogWidget.loginWidget.passwordNode.domNode.value = "";
			if (response instanceof Error) {
				var errorObj= {
					status: ioArgs.xhr.status
				};
				if (ioArgs.xhr.status == 403 && this._previousRequest.url.match(/^.*IWebUIInitializerRestService\/initializationData.*$/)) {
					// Capture the scenario where a user account does not belong to the groups required to access a requested resource
					// Provide a helpful error message rather than a blank "Loading..." screen.
					var errMsg = messages.noAccessWeb;
					alert(errMsg);
					errorObj.message = errMsg;
				}
				// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
				errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
				serviceResponseHandler.doErrorCallback(errorObj);
			}
			else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == "authfailed") {
				_loginPrompt(bindArgs, serviceResponseHandler, requestMethod, true, messages.loginFailed);
			}
			else {
				var dialogWidget = dijit.byId("com-ibm-team-repository-internal-LoginWidgetDialog");
				if (dialogWidget.queuedRequests != null) {
					while (dialogWidget.queuedRequests.length > 0) {
						var request = dialogWidget.queuedRequests.shift();
						if (request.requestMethod == "post") {
							xhr = dojo.rawXhrPost(request.bindArgs);
						}
						else {
							xhr = dojo.xhrGet(request.bindArgs);
						}
					}
				}
			}
        })
    };
    dojo.xhrPost(bindArgs);
}

/*
 * Assembles a URL, appending a query string to GET queries.  Handles both
 * non-secure and secure contexts.
 */
function _assembleURL(serviceRequest) {
	var method = serviceRequest.getMethod();
	var methodType = serviceRequest.getMethodType();
	var urlResource =
		method.substring(methodType.length, methodType.length + 1).toLowerCase() +
		method.substring(methodType.length + 1);
	
	// assemble basic url
	var url = _getServicePrefix() + serviceRequest.getServiceInterface() + "/" + urlResource;
	
	// if this is a GET request, append the query string
	if (methodType == "GET") {
		var tmpQueryString = serviceRequest.getQueryString();
		var queryString = tmpQueryString ? ("?" + tmpQueryString) : "";
		url += queryString;
	}
	return url;
}

/*
 * Returns the correct service prefix based on the browser's security
 * context (e.g. http or https).
 */
function _getServicePrefix() {
	return SERVICE_PREFIX;
}

// export for the sake of workitem.web, which depends on this (though it shouldn't)
com.ibm.team.repository.web.transport.TeamServerClient._getServicePrefix = _getServicePrefix;

//
// Internal demarshalling functions
//

function _demarshallResponse(evaldObj) {
	var returnValue= evaldObj["soapenv:Body"].response.returnValue;
	
	var obj= null;
	
	if (returnValue.values) {          // array of EMF objects 
		obj= returnValue.values;
	} else if (returnValue.value) {    // single EMF object
		obj= returnValue.value;
	} else if (returnValue.dataArgs) { // data query
		obj= _demarshallDataQuery(returnValue.dataArgs);
	}
	
	return obj;
}

function _demarshallDataQuery(srcRows) {
	var targetRows= [];
	for (var i=0; i<srcRows.length; i++) {
		var srcCols= srcRows[i].dataArgs;
		var targetCols= [];
		for (var j=0; j<srcCols.length; j++) {
			targetCols.push(srcCols[j].value);
		}
		targetRows.push(targetCols);
	}
	return targetRows;
}

})();