/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.Header");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.ui.internal.Login");
dojo.require("com.ibm.team.repository.web.ui.internal.Logout");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dijit.Menu");
dojo.require("com.ibm.team.repository.web.ui.internal.ProjectSelector");


/**
 * Provides the web UI's header area.
 */

var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
var InitializationData			= com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler 		= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var X_AUTH_MSG					= "X-com-ibm-team-repository-web-auth-msg";
var ProjectSelector				= com.ibm.team.repository.web.ui.internal.ProjectSelector;

dojo.declare("com.ibm.team.repository.web.ui.internal.Header", [dijit._Widget, dijit._Templated], {
	
	//
	// Module-private properties and methods
	//

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/Header.html"),
	
	showLoginPanel: false,
		
	/**
	 * Fill in the dynamic elements of the header nav.
	 */
	postCreate: function() {
		if(this.showLoginPanel){
			if (this._isInitDataLoaded()) {
				this._showPanel();
			} else {
				this._showPanelNoRepo();
			}
		}
		this._showProjectAreaSelector();
	},
	_showProjectAreaSelector: function() {
		this._selectWrapper.appendChild(new ProjectSelector({label: this.label, archived: this.archived, _projectAreas: this.projectAreas}).domNode);
	},
	_showPanel: function() {
		if (isGuest()) {
			this._showLoginPanel();
		} else {
			this._showLogoutPanel();
		}
	},
	_handleError: function() {
		
	},
	_showPanelNoRepo: function () {
		var bindArgs= {
	        url: net.jazz.ajax._contextRoot + "/auth/whoami",
	        content: { },
			handleAs: "json",
			headers: { },
			handle: dojo.hitch(this, function(response, ioArgs){
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status
					};
					// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
					errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
					serviceResponseHandler.doErrorCallback(errorObj);
				}
				else if (ioArgs.xhr.getResponseHeader(X_AUTH_MSG) == null) {
					this._showLoginPanel();
				}
				else {
					this._showLogoutPanel(ioArgs.xhr.getResponseHeader(X_AUTH_MSG));
				}
	        })
	    };
	    dojo.xhrGet(bindArgs);
	},

	_showLoginPanel: function() {
		this.loginPanelContainer.appendChild(new com.ibm.team.repository.web.ui.internal.Login().domNode );
	},
	
	_showLogoutPanel: function(loggedInUser) {
		this.loginPanelContainer.appendChild(new com.ibm.team.repository.web.ui.internal.Logout({ "loggedInUser": loggedInUser}).domNode );
	},
	
	_isInitDataLoaded: function() {
		if (InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer")) {
			return true;
		}
		return false;
	}
});

})();
