/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ActiveServicesView.html"),

	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");
	},	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView.superclass.postCreate.apply(this);
		this._fetchAndDisplayServiceInfo();
	},
	
	_fetchAndDisplayServiceInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayActiveServicesInfo", "_handleActiveServicesFetchError");
		ServerStatusClient.getActiveServiceInfo(srh)
	},

	_displayActiveServicesInfo: function(activeServiceInfo) {
		dojox.data.dom.removeChildren(this.activeServicesArea);
		 
		var servicesDiv = document.createElement("div");	
		var services = activeServiceInfo.services;
		//KMB TODO: sort service list
		if (services) {
			for (var i=0; i<services.length; i++) {
				this._appendActiveServiceSection(services[i], servicesDiv, activeServiceInfo.currentTime);
			}
		}
		this.activeServicesArea.appendChild(servicesDiv);
	},
	
	_appendActiveServiceSection: function(service, node, currentTime) {
		var elapsedTime = currentTime - service.startTime;
		elapsedTime = formatTimeFromMilliseconds(elapsedTime);
		elapsedTime = ", " + elapsedTime + this.messages.runningTime;
		
		var div = document.createElement("div");
		div.className = "right-area";
		
		var timeSpan = document.createElement("span");
		timeSpan.className = "running-time";
		timeSpan.innerHTML = elapsedTime;
		
		var requesterSpan = document.createElement("span");
		requesterSpan.className = this.messages.requester;
		requesterSpan.innerHTML = this.messages.by + service.userId;
		
		var serviceNameSpan = document.createElement("div");
		serviceNameSpan.className = "service-name";
		serviceNameSpan.innerHTML = service.service + "." + service.method;
		
		var serviceTitleDiv =  document.createElement("div");
		serviceTitleDiv.className = "service-title";
		
		div.appendChild(requesterSpan);
		div.appendChild(timeSpan); 
		serviceTitleDiv.appendChild(serviceNameSpan); 
		serviceTitleDiv.appendChild(div);
		
		var traceDiv = document.createElement("div");
		traceDiv.className = "service-trace";
		traceDiv.innerHTML = replaceNewLineWithPaddedBreaks(service.stacktrace, true);
		
		serviceTitleDiv._traceDiv = traceDiv;
		serviceTitleDiv._traceDiv.style.display = "none";
		
		serviceTitleDiv.onclick = function(){
			newDisplay = serviceTitleDiv._traceDiv.style.display === "block"
				? "none" : "block";
			serviceTitleDiv._traceDiv.style.display = newDisplay;
		}
		
		var activeService = document.createElement("div");
		activeService.className = "active-service";
		activeService.appendChild(serviceTitleDiv);
		activeService.appendChild(traceDiv);
	
		node.appendChild(activeService);
	},
		
	_handleActiveServicesFetchError: function(errorObj) {
		this._displayErrorMessage(this.messages.serviceError, errorObj);
	},	

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			var detailedReason = errorObj.status === 0 ? this.messages.unreachable : errorObj.message;
			message += dojo.string.substitute(this.messages.detailed, [ detailedReason ]); ;
		}
		this._displayMessage(message, "errorMsg");
	},
	
	// do not call directly
	_displayMessage: function(message, className) {
		dojox.data.dom.removeChildren(this.messageArea);
		this.messageArea.className= className;
		this.messageArea.style.display = "block"
		this.messageArea.appendChild(document.createTextNode(message));
	}, 
	
	_clearMessages: function () {
		dojox.data.dom.removeChildren(this.messageArea);
		this.messageArea.style.display = "none";
	}

});

})();