/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.number");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.internal.HoverPopup");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FileUploader");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ExpandingComponent");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");


//
// Aliases
//

var FileUploader = com.ibm.team.repository.web.ui.internal.admin.server.FileUploader;
var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
var formatCALTypeDescription = LIC_P.formatCALTypeDescription;
var getLicenseInfo = LIC_P.getLicenseInfo;
var getUsedLicenseInfo = LIC_P.getUsedLicenseInfo;
var getExpirationInfo = LIC_P.getExpirationInfo;
var sortCALs = LIC_P.sortCALs;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var postLicenseInfo = LIC_P.postLicenseInfo;
var removeChildren = dojox.data.dom.removeChildren;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var UPLOAD_URI = LIC_P.FILE_UPLOAD_URI;
var FLOATING_LICENSE_SERVICE_NAME = "com.ibm.team.repository.service.internal.license.LicenseService";
var ExpandingComponent = com.ibm.team.repository.web.ui.internal.admin.server.ExpandingComponent;

var FLOATING_SUPPORT_OPERATION_ID = "com.ibm.team.repository.support.floatingLicenses";
var ACTION_ID = "action=com.ibm.team.repository.admin.manageLicenses";
var DEFAULT_FOCUS = "server";
var TAB_DETAILS = [
	{labelKey: "serverTabLabel", focusTab: "server", link: "serverTab", content: "serverTabContent"},
	{labelKey: "floatingTabLabel", focusTab: "floating", link: "floatingTab", content: "floatingTabContent"}
];

var messages;
//
// Widget definition
//

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor", [dijit._Widget, dijit._Templated], {
	
	//
	// Class properties
	//
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseEditor.html"),
	
	//
	// Widget lifecycle methods
	//
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
		var floatingTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=floating'>";
		this.clientDescriptionIncludingLink = dojo.string.substitute(this.messages.clientDescription, [ floatingTabLink, "</a>" ]);
		var serverTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=server'>";
		this.noFloatingSupportIncludingLink = dojo.string.substitute(this.messages.noFloatingSupport, [ serverTabLink, "</a>" ]);
		this.floatingUploadWarningIncludingLink = dojo.string.substitute(this.messages.floatingUploadWarning, [ floatingTabLink, "</a>" ]);
	},	

	postCreate: function() {
		this.loadForms();
		this._setInfopopHeading(this.limitHeader, "limitHeader", "limitHeaderInfo");
		this._setInfopopHeading(this.includedHeader, "includedHeader", "includedHeaderInfo");
		this._setInfopopHeading(this.totalHeader, "totalHeader", "totalHeaderInfo");
		this._setInfopopHeading(this.assignedHeader, "assignedHeader", "assignedHeaderInfo");
		this._setInfopopHeading(this.availableHeader, "availableHeader", "availableHeaderInfo");
		this._setInfopopHeading(this.floatLimitHeader, "floatLimitHeader", "floatLimitHeaderInfo");
		this._setInfopopHeading(this.floatIncludedHeader, "floatIncludedHeader", "floatIncludedHeaderInfo");
		this._setInfopopHeading(this.floatTotalHeader, "floatTotalHeader", "floatTotalHeaderInfo");
		this._setInfopopHeading(this.floatPurchasedHeader, "floatPurchasedHeader", "floatPurchasedHeaderInfo");
				
		if (!this.focusTab){
			this.focusTab = DEFAULT_FOCUS;
		}	
		this.loadTabs();
	}, 
	
	/**
	 * Fetches the most recent saved license information from the server and
	 * resets the editor with this information (long-op).
	 */
	reload: function() {
		if(this._isDirty && !confirm(this.messages.refreshDirty)){
			return;
		}
		getWorkbench().showMessage(this.messages.loadingLicenseInfo);
		var self = this;
		var o = {
			s: function(licenseInfo) {
				getWorkbench().hideMessageArea();
				self.reset(licenseInfo);
			},
			f: function(err) {self.cbRemoteError(err);}
		}
		getLicenseInfo(new SRH(o, "s", "f"));
	},
	
	rebind: function() {
		this.refreshServerTable();
		this.refreshCALTable();
		this.refreshFloatingCALTable();
	},
	
	/**
	 * Resets both the object and UI state of this editor, based on the
	 * <code>licenseInfo</code> parameter (short-op).
	 */
	reset: function(licenseInfo) {
		// reset object state
		this.initialLicenseInfo = this.licenseInfo = licenseInfo;
		this.usedLicenseInfo = null;
		this._uploadedCALFiles = {};
		this._uploadedServerFile = null;
		this.lastFloatingCount = licenseInfo.floatingLicenseTypes && licenseInfo.floatingLicenseTypes.length ? licenseInfo.floatingLicenseTypes.length : 0;
		this.isDirtyLicenseInfo = false;
		this.resetConfigurationSection();
		
		// reset UI state
		this.hideMessageArea();
		this.checkDirty();
		this.focusTabs();
		this.rebind();
		this.saveButton.disabled = true;
		this.newCALUploadForm.setInlineMessage("", "noDisplay");
		this.newFloatCALUploadForm.setInlineMessage("", "noDisplay");
		
		var self = this;
		var floatHelper = {
			yes: function(status) {
				self.addLicenseUsageDetails();
			},
			no: function(s) {
				self.usedLicenseInfo = null;
			},
			error: function(err) {
				self.cbRemoteError(err);
			}
		}
		
		LIC_P.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
	},
	
	/**
	 * Fetches the detailed breakdown of the license usage and updates the UI.
	 * This method will only success if floating licenses are supported on the server
	 */
	addLicenseUsageDetails: function() {
		this.licenseInfo.operationCheckResults = [
			{"operationId": FLOATING_SUPPORT_OPERATION_ID, "allowed": true}
		];
		var self = this;
		var floatingLicenseInfoHandler = {
			sucess: function(info) {
				self.updateUsedLicenseData(info);			
			},
			failure: function(err) {
				self.usedLicenseInfo = null;
				self.cbRemoteError(err);
			}
		}
		getUsedLicenseInfo(new SRH(floatingLicenseInfoHandler, "sucess", "failure"));
	},
	
	/**
	 * This is called when addition details of the used
	 * licenses on the server.  It provides a breakdown
	 * between floating and non-floating use.  This method
	 * might not be called if the underlying server does not 
	 * support floating licenses
	 */
	updateUsedLicenseData: function(usedLicenseInfo) {
		this.usedLicenseInfo = usedLicenseInfo;
		
		if (this.licenseInfo)
			this.refreshServerTable();
	},
	
	/**
	 * Saves the current state of the editor (long-op).
	 */
	save: function() {
		this.saveButton.disabled = true;
		getWorkbench().showMessage(this.messages.savingLicenseInfo);
		if (this.configureLicenseServer) {
			var invokedSave = this.configureLicenseServer._saveHandler();
			// nothing to save in config 
			if (!invokedSave) {
				this.saveEditor();
			}
			// otherwise the successful save is connected to saveEditor		
		} else {
			this.saveEditor();
		}
	},
	
	saveEditor: function () {
		var args = dojo.mixin(this.getEditorData(), {commit: true});
		postLicenseInfo(new SRH(this, "cbSave", "cbSaveError"), args);
	}, 
	
	/**
	 * Success callback from 'Save' action; resets editor based on newly
	 * persisted state.
	 */
	cbSave: function(licenseInfo) {
		this.reset(licenseInfo);
		function f() {
			getWorkbench().hideMessageArea();
		}
		dojo.publish(com.ibm.team.repository.web.internal.events.licenseChange, [licenseInfo]);
		this.showEventOutcomeMessage("ok", this.messages.saveComplete, f, 1500);
	},
	
	cbSaveError: function(errObj) {
		this.saveButton.disabled = false;
		this.cbRemoteError(errObj, this.messages.saveError);
	},
	
	/**
	 * Called by Admin Web UI when this cached widget is displayed; update
	 * the form's backing data to make sure we pick up any changes to user
	 * assignments.
	 */
	show: function(args) {
		if (this.focusTab && args.focusTab && this.focusTab != args.focusTab) {
			this.focusTab = args.focusTab
			this.focusTabs();
			return;
		}
		this.focusTab = args.focusTab ? args.focusTab : DEFAULT_FOCUS;
		
		if(this._isDirty) {
			this.fetchToBeState();
		} else {
			this.reload();
		}
	},
	
	//
	// UI data binding and querying
	//
	
	loadForms: function() {
		var formName1 = "newCALUploadForm";
		this[formName1] = this.createForm("CAL", this.messages.calTypelabel, formName1);
		
		var formName2 = "serverUploadForm";
		this[formName2]	= this.createForm("server", this.messages.serverTypelabel, formName2);
		
		var formName3 = "newFloatCALUploadForm";
		this[formName3] = this.createForm("CAL", this.messages.floatingTypelabel, formName3);		
	},
	
	
	// KMB: since this is all static, I tried the same DOM in the template, but the CSS 
	// didn't work to create the tab outlines.  Need to investigate why.  For now, it's in JavaScript
	loadTabs: function() {
		var self = this;
		dojo.forEach(TAB_DETAILS, function(tab) {
			var link = document.createElement("a");
			link.tabId = tab.focusTab;
			link.href = "#" + ACTION_ID + "&focusTab=" + tab.focusTab;
			link.className = "tab";
			var topWrapper = document.createElement("div");
			topWrapper.className = "topWrapper";
			var topBorder = document.createElement("div");
			topBorder.className = "topBorder";
			topWrapper.appendChild(topBorder);
			var corners = document.createElement("div");
			corners.className = "corners";
			topWrapper.appendChild(corners);
			link.appendChild(topWrapper);
			var innerWrapper = document.createElement("div");
			innerWrapper.className = "innerWrapper";
			var title = document.createElement("div");
			title.className = "title";
			title.appendChild(document.createTextNode(self.messages[tab.labelKey]));
			innerWrapper.appendChild(title);
			link.appendChild(innerWrapper);
			self.tabSection.appendChild(link);
			self[tab.link] = link;
			self.connect(link, "onclick", function (e) {
				e.target.blur();
			});
		});
		this.focusTabs();
	},
	
	createConfigurationSection: function() {
		removeChildren(this.floatingServerProperty);
		this.configureLicenseServer = new ExpandingComponent({
			implementationClassName: FLOATING_LICENSE_SERVICE_NAME
		});
		this.connect(this.configureLicenseServer, "notifyStatusChange", "updateConfigState");
		this.connect(this.configureLicenseServer, "_submitConfigCallback", "saveEditor");
		this.connect(this.configureLicenseServer, "loaded", "displayCountedUserBreakdown");
		this.floatingServerProperty.appendChild(this.configureLicenseServer.domNode);
	},
	
	resetConfigurationSection: function() {
		if (this.configureLicenseServer){
			this.configureLicenseServer.restore();
		}
	},
	
	createForm: function(typeCode, typeLabel, formName) {
		var self = this;
		var formLabel = dojo.string.substitute(self.messages.upload, [typeLabel])+ ": ";
		var args = {
			formElements: [{name: "licenseType", value: typeCode}],
			formLabel: formLabel,
			onBeginUpload: function() { getWorkbench().showMessage(self.messages.uploadingFile); },
			onUpload: function(o) { self.cbUpload(typeCode, typeLabel, formName, o.fileName); },
			onUploadFailure: function(errObj){ self.cbUploadFailure(typeLabel, errObj); },
			postURI: UPLOAD_URI
		}
		return new FileUploader(args, this[formName]);
	}, 
	
	/**
	 * Returns the current data associated with the editor needed to call
	 * <code>postLicenseInfo()</code>.
	 */
	getEditorData: function() {
		var calFiles = [];
		for (var fileName in this._uploadedCALFiles) {
			calFiles.push(fileName);
		}
		var args = {
			serverFile: this._uploadedServerFile,
			calFiles: calFiles,
			maxCounts: [],
			operationIds: [FLOATING_SUPPORT_OPERATION_ID]
		};
		
		// if this is not a trial, possibly update the purchased counts
		var inputs = this.CALInfoTable.getElementsByTagName("input");
		dojo.forEach(inputs, function(input){
			if(!input.disabled) {
				args.maxCounts.push({id: input.__jazzCALId, max: toInt(input.value)});
			}
		});
		// KB to opt
		inputs = this.FloatCALInfoTable.getElementsByTagName("input");
		dojo.forEach(inputs, function(input){
			if(!input.disabled) {
				args.maxCounts.push({id: input.__jazzCALId, max: toInt(input.value)});
			}
		});

		return args;
	},
	
	/**
	 * Refreshes the Client Access License table based on the current value of
	 * <code>this.licenseData.types</code>.
	 */
	refreshCALTable: function() {
		var tbody = this.CALInfoTable;
		var self = this;
		var inputs = this.CALInfoTable.getElementsByTagName("input");
		dojo.forEach(inputs, function(input) {
			if(input.__jazzChgEvt){
				self.disconnect(input.__jazzChgEvt);
			}
		});
		removeChildren(tbody);
		
		// normalize data and create table columns
		var types = this.licenseInfo.types || [];
		types = sortCALs(types);
		var isTrial = this.licenseInfo.serverLicense.expirationTime ? true : false;		
		this._setPurchasesOrTrialTableHeader(this.purchasedOrTrialHeader);
		dojo.forEach(types, function(type){
			var row = d.ce("tr");
			tbody.appendChild(row);
			// deal with disabled CALs
			var isDisabled = type.disabledReason != null;
			if(isDisabled) {
				row.className = "disabledCAL";
			}
			
			var limit = getCountOrUnlimited(type.max);			
			var included = getCountOrUnlimited(type.free);
			var total = getCountOrUnlimited(type.total);
			
			// set displayable counts depending on whether the product is a
			// permanent edition or a trial
			if (isTrial) {
				var available = type.max === -1 ? self.messages.unlimited : type.max - type.used;
				var purchase = type.max === -1 ? self.messages.notApplicable : type.max - type.free;
				var total = getCountOrUnlimited(type.max);
			} else {
				var available = type.total === -1 ? self.messages.unlimited : type.total - type.used;
				var purchase = type.free === -1 ? self.messages.notApplicable : type.total - type.free;
			}
			
			// create the columns for the given CAL
			var nameLink = self._getInfopopLink(type.name, formatCALTypeDescription(type));
			var nodes = [
				type.productName,
				type.editionName + " (" + type.variantName + ")",
				nameLink,
				limit,
				included,
				// purchased or trial count
				purchasedOrTrialCol(self, type, isTrial, purchase, /*isFloating */ false),
				total,
				getUsedLink(type),
				available,
				isDisabled ? dojo.string.substitute(self.messages.deactivatedReason, [ type.disabledReason ]) : self.messages.activated
			];			
			dojo.forEach(nodes, function(node) {
				col(row, node);
			});
		});
	},	
	
	_getInfopopLink: function(label, info, className) {
		var hoverLink = d.ce("a");
		hoverLink.href = "#";
		hoverLink.onclick = function(){
			return false;
		};
		hoverLink.innerHTML = label;
		if (className) {
			hoverLink.className = className;
		} else {
			hoverLink.className = "infopop";
		}
		var tooltip = new HoverPopup({
			connectNode: [hoverLink],
			label: info,
			showDelay: 300,
			orientation: "R"
		}, d.ce("span"));
		
		return hoverLink;
	},

	_setInfopop: function (headerCell, label, info, className){
		removeChildren(headerCell);
		var hoverLink = this._getInfopopLink(label, info, className);
		headerCell.appendChild(hoverLink);
	},

	_setInfopopHeading: function (headerCell, labelKey, infoKey) {
		this._setInfopop(headerCell, this.messages[labelKey], this.messages[infoKey], "infopop infopop-faded");
	},
	
	_setPurchasesOrTrialTableHeader: function (headerCell) {
		var isTrial = this.licenseInfo.serverLicense.expirationTime ? true : false;
		var labelKey = isTrial ? "trialHeader" : "purchasedHeader";
		var infoKey = isTrial ? "trialHeaderInfo" : "purchasedHeaderInfo";
		this._setInfopopHeading(headerCell, labelKey, infoKey);
	},
	
	/**
	 * Refreshes the Floating Client Access License table based on the current value of
	 * <code>this.licenseData.floatingLicenseTypes</code>.
	 */
	refreshFloatingCALTable: function() {
		// KB to opt this function
	
		// clear current CAL table values
		var tbody = this.FloatCALInfoTable;
		var self = this;
		var inputs = this.FloatCALInfoTable.getElementsByTagName("input");
		dojo.forEach(inputs, function(input) {
			if(input.__jazzChgEvt){
				self.disconnect(input.__jazzChgEvt);
			}
		});
		removeChildren(tbody);
	
		// normalize data and create table columns
		var types = this.licenseInfo.floatingLicenseTypes || [];
		dojo.toggleClass(this.FloatCALInfo, "noDisplay", !this.licenseInfo.floatingLicenseTypes);
		if (this.licenseInfo.floatingLicenseTypes) {
			types = sortCALs(types);
			dojo.forEach(types, function(type){
				var row = d.ce("tr");
				tbody.appendChild(row);
				// deal with disabled CALs
				var isDisabled = type.disabledReason != null;
				if (isDisabled) {
					row.className = "disabledCAL";
				}
				
				// trial mode of server has no effect on counts for floating
				var limit = getCountOrUnlimited(type.max);			
				var included = getCountOrUnlimited(type.free);
				var total = getCountOrUnlimited(type.total);				
				var purchase = type.free === -1 ? self.messages.notApplicable : type.total - type.free;

				// create the columns for the given CAL
				var nameLink = d.ce("a");
				nameLink.href = "#";
				nameLink.onclick = function(){
					return false;
				};
				nameLink.innerHTML = type.name;
				nameLink.className = "infopop";
				var description = formatCALTypeDescription(type);
				var tooltip = new HoverPopup({
					connectNode: [nameLink],
					label: description,
					showDelay: 300,
					orientation: "R"
				}, d.ce("span"));
				var nodes = [
					type.productName, 
					type.editionName + " (" + type.variantName + ")", 
					nameLink, 
					limit, 
					included,  
					// purchased or trial count
					purchasedOrTrialCol(self, type, /* isTrial */ false, purchase, /* isFloating*/ true ), 
					total, 
					isDisabled ? dojo.string.substitute(self.messages.deactivatedReason, [ type.disabledReason ]) : self.messages.activated,
					getIssuedLink(type)
				];
				dojo.forEach(nodes, function(node){
					col(row, node);
				});
			});
		}
	},
		
	/**
	 * Refreshes the Server License information table based on the current value
	 * of <code>this.licenseData.serverLicense</code>.
	 */
	refreshServerTable: function() {
		var serverLicense = this.licenseInfo.serverLicense;
		removeChildren(this.serverInfoTable);
		var tbody = this.serverInfoTable;
		var isTrial = serverLicense.expirationTime ? true : false;
		if(isTrial) {
			var expirationInfo = getExpirationInfo(serverLicense.expirationTime);
		}
		var isDisabled = serverLicense.disabledReason || (isTrial && expirationInfo.isExpired);
		function row(propRow, className) {
			var row = d.ce("tr");
			row.className = isDisabled ? "disabledServer" : "";
			tbody.appendChild(row);
			var td = d.ce("td");
			row.appendChild(td);
			td.innerHTML = propRow.n;
			td.className = "propColumn";
			var td = d.ce("td");
			row.appendChild(td);
			td.innerHTML = propRow.v;
			if(className) {
				td.className = className;
			}
		}
		var maxUsers = serverLicense.maxContributors !== -1 ? serverLicense.maxContributors : this.messages.unlimited;
		var editionLabel = serverLicense.editionName ? serverLicense.editionName : "";
		editionLabel += serverLicense.variantName ? " (" + serverLicense.variantName + ")" : ""; 
		
		var usedLicenseText = serverLicense.usedContributors;
		if (this.usedLicenseInfo) {
			if (this.configureLicenseServer && this.configureLicenseServer.hasURI()){
				var numFloating = this.usedLicenseInfo.countedActiveLeases ? this.usedLicenseInfo.countedActiveLeases : 0;
				var activeLabel = dojo.string.substitute(this.messages.acquiredLinkLabel, [ numFloating ]);
				var activeFloatingLink = "<a href=\"#action=com.ibm.team.repository.activeFloatingLicenses\">" + activeLabel + "</a>" ;
				var assigned = this.usedLicenseInfo.totalUsedCount - numFloating;
				var assignedLabel = dojo.string.substitute(this.messages.assignedLinkLabel, [ assigned ]);
				var assignedLink = "<a href=\"#action=com.ibm.team.repository.manageUserCALs\">" + assignedLabel + "</a>" ;
				
				usedLicenseText = this.usedLicenseInfo.totalUsedCount + " (" + assignedLink + ", " + activeFloatingLink + ")";
			} 
			
			if (findOperationCheckResults(this.licenseInfo, FLOATING_SUPPORT_OPERATION_ID) == false) {
				var warningSection = "<span class=\"inline-message WARNINGcolor\">" + this.messages.noFloatSupport + "</span>";
				usedLicenseText  = usedLicenseText + warningSection;
			}  
			
			// display UI to set config props to point to a lic server
			if (!this.configureLicenseServer) {
				this.createConfigurationSection();
			}
			
		} 
		
		dojo.toggleClass(this.floatingServerProperty, "noDisplay", !this.usedLicenseInfo);
		dojo.toggleClass(this.floatingAllowedContent, "noDisplay", !this.usedLicenseInfo);
		dojo.toggleClass(this.floatingNotAllowedContent, "noDisplay", this.usedLicenseInfo);
		
		var isFloatingServer = dojo.getObject("licenseInfo.floatingSetupStatus.server", false, this);
		if (this.usedLicenseInfo && isFloatingServer){
			dojo.publish(com.ibm.team.repository.web.internal.events.floatingLicenseChange, [{
				actingAsFloating: true
			}]);
		}
		
		var rows = [
			{n: this.messages.serverProduct, v: serverLicense.productName},
			{n: this.messages.serverEdition, v: editionLabel},
			{n: this.messages.serverDescription, v: serverLicense.description},
			{n: this.messages.maximumUsers, v: maxUsers},
			{n: this.messages.countedUsers, v: usedLicenseText}
		];
		dojo.forEach(rows, function(propRow){
			row(propRow);
		});

		var className = null;
		if(serverLicense.disabledReason) {
			var statusString = serverLicense.disabledReason;
		} else {
			var statusString = !isTrial ? this.messages.permanent : expirationInfo.isExpired ? this.messages.expiredTrial : this.messages.trial;
		}
		row({n: this.messages.serverStatus, v: statusString}, isDisabled ? "expired" : null);
		
		if(isTrial) {
			if(expirationInfo.isExpired) {
				var rowObj = {n: this.messages.expiredOn, v: expirationInfo.expirationDateString + " (" + expirationInfo.daysSinceExpirationString + ")"};
			} else {
				var rowObj = {n: this.messages.expiresOn, v: expirationInfo.expirationDateString + " (" + expirationInfo.daysRemainingString + ")"};
			}
			row(rowObj, expirationInfo.isExpired ? "expired" : null);
		}
	},	
	
	displayCountedUserBreakdown: function() {
		this.refreshServerTable();
	},	
	
	//
	// UI state manipulation methods
	//
	
	focusTabs: function() {
		var self = this;
		dojo.forEach(TAB_DETAILS, function(tab) {
			dojo.toggleClass(self[tab.content], "noDisplay", self.focusTab != tab.focusTab);
			var link = self[tab.link];
			dojo.toggleClass(link, "selected", self.focusTab === tab.focusTab);
			link.disabled = self.focusTab === tab.focusTab;
		});
	},
	
	markDirtyLicenseInfo: function() {
		this.isDirtyLicenseInfo = true;
		this.checkDirty();
	},
	
	checkDirty: function() {
		var dirty = false; 
		if (this.configureLicenseServer && this.configureLicenseServer.configStatusIsDirty()){
			dirty = true;
		}
		if (this.isDirtyLicenseInfo){
			dirty = true;
		}
		this._isDirty = dirty;
		this.updateDirtyUI();
	},
	
	updateDirtyUI: function() {
		this.dirtyMarker.innerHTML = this._isDirty ? "*" : "";
	},

	
	clearDirty: function() {
		this.dirtyMarker.innerHTML = "";
		this._isDirty = false;
	},
	
	//
	// Utility methods
	//
	
	/**
	 * General purpose error callback for remote methods.
	 */
	cbRemoteError: function(errorObj, msg) {
		if(!msg) {
			msg = this.messages.serverError;
		}
		if(errorObj.status === 0) {
			msg += this.messages.serverUnreachable;
		} else {
			msg += " (" + errorObj.message + ")";
		}
		this.showEventOutcomeMessage("error", msg);
	},
	
	//
	// UI Status and State reporting
	//
	
	updateSaveButton: function() {
		var configBlocksSave = false;
		if (this.configureLicenseServer && this.configureLicenseServer.configStatusBlocksSave()) {
			configBlocksSave = true;
		}
			
		this.saveButton.disabled = !this._isDirty || this.status.errors.length > 0  || configBlocksSave;
	},
	
	updateConfigState: function() {
		this.checkDirty();
		this.validate();
	},
		
	showEventOutcomeMessage: function(type, message, cb, duration) {
		// doesn't do anything with type?
		if(cb){
			setTimeout(cb, duration || 3000);
		} else if (duration) {
			setTimeout(function(){getWorkbench().hideMessageArea();}, duration);
		}
		getWorkbench().showMessage(message);
	},
	
	updateStatusMessage: function() {
		var hasErrors = this.status.errors.length > 0;
		var hasWarnings = this.status.warnings.length > 0;
		var hasPendingAction = this.status.pending.length > 0;
		if(hasErrors || hasWarnings) {
			if (hasErrors) {
				this.showMessageArea("ERROR", this.status.errors);
			} else {
				this.showMessageArea("WARNING", this.status.warnings);
			}
		} else if (this._isDirty) {
			if (hasPendingAction) {
				this.showMessageArea("INFO", this.status.pending);
			} else {
				this.showMessageArea("INFO", [this.messages.pendingChangesInfo]);
			}
		} else {
			this.hideMessageArea();
		}
	},
	
	showMessageArea: function(type, messageNodes) {
		removeChildren(this.messageArea);
		if(!dojo.isArrayLike(messageNodes)) {
			messageNodes = [messageNodes];
		}
		this.messageArea.className = "messageArea " + type + " " + type + "color";
		var self = this;
		dojo.forEach(messageNodes, function(node){
			if(dojo.isString(node)) {
				node = document.createTextNode(node);
			}
			self.messageArea.appendChild(node);
			self.messageArea.appendChild(document.createElement("br"));
		});
		this.messageArea.style.visibility = "visible";
	},
	
	hideMessageArea: function(cb) {
		this.messageArea.style.visibility = "hidden";
		this.messageArea.innerHTML = "";
	},
	
	//
	// General UI code
	//
	
	//
	// Validation logic and UI - validation can be triggered in two ways:
	// 1) User uploads a file (cbUpload calls fetchToBeState)
	// 2) User enters a new purchased count for a CAL (onPurchasedCountKeyUp)
	//
	
	fetchToBeState: function() {
		getWorkbench().showMessage(this.messages.validating);
		var args = dojo.mixin(this.getEditorData(), {commit: false});
		postLicenseInfo(new SRH(this, "cbFetchToBeState", "cbRemoteError"), args);
	},
	
	cbFetchToBeState: function(licenseInfo) {
		getWorkbench().hideMessageArea();
		this.licenseInfo = licenseInfo;
		this.rebind();
		this.validate();
		if (licenseInfo.floatingLicenseTypes && licenseInfo.floatingLicenseTypes.length != this.lastFloatingCount) {
			this.newCALUploadForm.setInlineMessage(this.floatingUploadWarningIncludingLink, "uploadWarning");
			if (this.lastFloatingCount === 0) {
				this.newFloatCALUploadForm.setInlineMessage(this.messages.convertToFloatingWarning, "uploadWarning");
			}
		}
	},
	
	onPurchasedCountKeyUp: function(e) {
		var input = e.target;
		if(!input || input.__jazzOrigValue === undefined) {
			return; // early return
		}
		
		if(input.value !== input.__jazzOrigValue) {
			this.markDirtyLicenseInfo();
		}
		// run a validation after the user stops typing for half a second
		if(this._calKeyStrokeTimout){
			clearTimeout(this._calKeyStrokeTimout);
		}
		this._calKeyStrokeTimout = setTimeout(dojo.hitch(this, this.validate), 500);
	},
	
	/**
	 * Validate does the following:
	 * 
	 * 1) Compares the state of the CALs in the dirty editor vs. the CALs'
	 *    original state. The purpose of this validation is to generate warnings
	 *    if the user is about to disable users that were previously enabled.
	 * 2) Ensures that editable CAL 'purchased' counts are >= 0 and less than
	 *    the maximum purchasable count for the CAL.
	 *    
	 * 3) Validates the License Server Configuration property value.
	 */
	validate: function() {
		var warnings = this.validateCALStates();
		warnings = warnings.concat(this.validateFloatingLicenseChanges());
		
		var errors = this.validateServerLicense();
		errors = errors.concat(this.validateCALPurchaseCounts());
		
		if (this.configureLicenseServer) {
			warnings = warnings.concat(this.configureLicenseServer.getConfigWarnings());
			errors = errors.concat(this.configureLicenseServer.getConfigErrors());
			var pending = this.configureLicenseServer.getConfigPendingStatus();
		}
		
		// CAL purchase counts can have errors, 
		// CAL state changes can produce warnings,
		// Server License Configuration can produce errors or warnings.
		this.status = {
			errors: errors,
			warnings: warnings, 
			pending: pending || []
		};
		
		this.updateSaveButton();
		this.updateStatusMessage();
	},
	
	validateFloatingLicenseChanges: function() {
		var floatingIssues = false;
		var supportsFloating = findOperationCheckResults(this.licenseInfo, FLOATING_SUPPORT_OPERATION_ID);
		
		if (this.usedLicenseInfo && this.usedLicenseInfo.activeLeases) {
			var numActiveLeases = this.usedLicenseInfo.activeLeases.length;
			if (numActiveLeases > 0) {
				if (!supportsFloating) {
					floatingIssues = true;
				}
			}
		}
		
		if (this.usedLicenseInfo && this.initialLicenseInfo.floatingLicenseTypes) {
			var numInstalledFloatingCALs = this.initialLicenseInfo.floatingLicenseTypes.length;
			if (numInstalledFloatingCALs > 0) {
				if (!supportsFloating) {
					floatingIssues = true;
				}
			}
		}

		var warnings = [];
		if (floatingIssues) {
			warnings.push(this.messages.noFloatMismatch);
		}
		if (this.lastFloatingCount === 0 && this.licenseInfo.floatingLicenseTypes && this.licenseInfo.floatingLicenseTypes.length > 0){
			warnings.push(this.messages.convertToFloatingWarning);
		}
		
		return warnings;
	},
	
	validateCALStates: function() {
		var warnings = [];
		var origGood = {};
		var origTypes = this.initialLicenseInfo.types;
		dojo.forEach(origTypes, function(type){
			if(!type.disabledReason) {
				origGood[type.id] = true;
			}
		});
		var dirtyTypes = this.licenseInfo.types;
		var self = this;
		dojo.forEach(dirtyTypes, function(type){
			if(type.disabledReason && origGood[type.id]) {
				warnings.push(dojo.string.substitute(self.messages.pendingCalDeactivation, [ type.name ]));
			}
		});
		return warnings;
	},
	
	validateCALPurchaseCounts: function() {
		var errors = [];
		var isValid = true;
		
		var inputs = this.CALInfoTable.getElementsByTagName("input"); 
		for(var i=0; i<inputs.length; i++) {
			if(inputs[i].disabled) {
				continue;
			}
			var result = validateCALInput(inputs[i]);
			if(!result.valid) {
				errors = [this.messages.purchaseCountError];
			}
		}
		// KB to opt here
		
		inputs = this.FloatCALInfoTable.getElementsByTagName("input"); 
		for(var i=0; i<inputs.length; i++) {
			if(inputs[i].disabled) {
				continue;
			}
			var result = validateCALInput(inputs[i]);
			if(!result.valid) {
				errors = [this.messages.purchaseCountError];
			}
		}
		return errors;
	},
	
	validateServerLicense: function() {
		var errors = [];
		if (this.licenseInfo.serverLicense && this.licenseInfo.serverLicense.disabledReason){
			errors = [this.messages.serverLicenseError];
		}
		return errors;
	},
	
	//
	// Upload-related methods
	//
	
	cbUpload: function(typeCode, typeName, widgetPropName, fileName) {
		this.markDirtyLicenseInfo();
		if(typeCode === "CAL") {
			this._uploadedCALFiles[fileName] = true;
		} else {
			this._uploadedServerFile = fileName;
		}
		this.showEventOutcomeMessage("ok", dojo.string.substitute(this.messages.successfulUpload, [ typeName ]), dojo.hitch(this, this.fetchToBeState), 1000);
	},
	
	cbUploadFailure: function(typeName, errObj) {
		this.showEventOutcomeMessage("error", dojo.string.substitute(this.messages.errorUpload, [ typeName]) + errObj.errorMessage);
	}
});

//
// Utility functions
//

var d = document;
d.ce = document.createElement;

function clearMessageTimeout(ctxObj, timeoutProp) {
	var timeoutObj = ctxObj[timeoutProp];
	clearTimeout(timeoutObj);
	ctxObj[timeoutProp] = null;
}

function col(row, node) {
	if(node === null || node === undefined) {
		node = "-";
	}
	var td = d.ce("td");
	var isNumber = typeof node == 'number' || node instanceof Number;
	var isNumberCol = isNumber || node === messages.notApplicable || node === messages.unlimited;
	if(isNumberCol) {
		td.className = "num";
	}
	if(isNumber) {
		node = dojo.number.format("" + node);
	}
	var isArray = !dojo.isString(node) && dojo.isArrayLike(node);
	if(node.tagName === "A" && node.className === "num") {
		td.className = "num";
	} else if (isArray) {
		td.className = "num inputCol";
	}
	row.appendChild(td);
	var nodeArr = isArray ? node : [node];
	dojo.forEach(nodeArr, function(node){
		if(dojo.isString(node)) {
			node = document.createTextNode(node);
		}
		td.appendChild(node);
	});
}

function purchasedOrTrialCol(widget, type, isTrial, purchasedOrTrial, isFloatingForm) {
	var input = d.ce("input");
	var msgDiv = d.ce("div");
	input.className = "num";
	input.value = input.__jazzOrigValue = "" + purchasedOrTrial;
	input.__jazzCALId = type.id;
	// works for all cases except bogus data max a number and free -1, yields max purchase max + 1
	input.__jazzMaxPurchasable = type.max === -1 ? Infinity : type.max - type.free;

	input.__jazzMsgArea = msgDiv;
	if (LIC_P.allowsPurchaseCountUpdates(type, isTrial, isFloatingForm)) {
		input.__jazzChgEvt = widget.connect(input, "onchange", widget.validate);
	} else {
		input.disabled = true;
	}
	return [input, msgDiv];
}

function getCountOrUnlimited(count) {
	return count === -1 ? messages.unlimited : count; 
}

function getCountString(count) {
	return count === -1 ? messages.unlimited : (dojo.number.format(count)); 
}

function getUsedLink(calType) {
	if(calType.used === 0) {
		return 0;
	} else {
		var a = document.createElement("a");
		a.href = "#action=com.ibm.team.repository.manageUserCALs&licenseType=" + calType.id;
		a.innerHTML = calType.used;
		a.className = "num";
		return a;
	}
}

function getIssuedLink(calType) {
	var a = document.createElement("a");
	a.href = "#action=com.ibm.team.repository.admin.floatingLicenseUsage&licenseId=" + calType.id;
	a.innerHTML = messages.viewIssued;
	return a;
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

var NUM_REGEX = /[0-9]+/g;

function toInt(text) {
	var match = text.match(NUM_REGEX);
	return match && parseInt(text, 10);
}

function validateCALInput(input) {
	var numVal = toInt(input.value);
	if(numVal !== null && numVal <= input.__jazzMaxPurchasable && numVal >= 0) {
		isValid = true;
		input.__jazzMsgArea.innerHTML = "";
	} else {
		var isValid = false;
		var invalidMsg = dojo.string.substitute(messages.invalidCount, [ input.__jazzMaxPurchasable ]);
		input.__jazzMsgArea.innerHTML = invalidMsg;
	}
	var node = input;
	while(node.tagName !== "TR") {
		if(node.tagName === "TD") {
			node.className = isValid ? "num inputCol" : "inputCol";
		}
		node = node.parentNode;
	}
	if(isValid) {
		dojo.removeClass(node, "hasError");
	} else {
		dojo.addClass(node, "hasError");
	}
	return {
		changed: input.value !== input.__jazzOrigValue,
		valid: isValid
	};
}

function findOperationCheckResults(licenseInfo, operationId) {
	var result = null;
	if (licenseInfo.operationCheckResults) {
		dojo.forEach(licenseInfo.operationCheckResults, function(operationResult) {
			if (operationResult.operationId == operationId) {
				result = operationResult.allowed;
			}
		})
	}
	return result;
}

})();