/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.WizardPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.EnableEmail");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.SetupUsers");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardNavigation");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardStepList");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardReview");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardIntro");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardPageMessages");


var checkNotEmpty 	= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var WizardStepList = com.ibm.team.repository.web.ui.internal.wizard.WizardStepList;


var PlatformUI     = net.jazz.ajax.ui.PlatformUI;
var ActionRegistry = null; // bind to ActionRegistry later

var PREFIX = "com.ibm.team.repository.wizard.";
var SETUP_USERS = "setupUsers";

// 1) add a new action
// 2) add a new method at the bottom similar to showReview and that method name corresponds to the shortId

var ACTIONS = [
	{label: "Introduction", shortId: "showIntro", active: true}, 
	{label: "Configure Database", shortId: "configureDatabase", active: false, requiresNextApproval: true},
	{label: "Enable E-mail Notification", shortId: "enableEmail", active: false, requiresNextApproval: true}, 
	{label: "Setup User Registry", shortId: SETUP_USERS, active: false, requiresNextApproval: true},
	{label: "Summary", shortId: "showReview", active: false}
];
var ACTION_MAP = {};
dojo.forEach(ACTIONS, function(action){
	ACTION_MAP[action.shortId] = action;
});

function getStepForAction (shortId) {
	for (var i=0; i < ACTIONS.length; i++) {
		if(ACTIONS[i].shortId === shortId) {
			return i;
		};
	}
	return 0;
}

function getNavigationLink(action, title) {
	checkNotEmpty(action, "action");
	checkNotEmpty(title, "title");
	
	var aElement = document.createElement("a");
	
	aElement.href = "#action=" + action;
	
	aElement.appendChild(document.createTextNode(title));
	
	return aElement;
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

var messages; 

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.WizardPage",  [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/WizardPage.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardPageMessages");
	},

	postCreate: function() {
		WizardPage.superclass.postCreate.apply(this);

		this._stepList = new com.ibm.team.repository.web.ui.internal.wizard.WizardStepList({prefix: PREFIX});
		this._stepSection = this._stepList._wizardSteps;
						
		// register actions and create hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		for (var i=0; i < ACTIONS.length; i++) {
			var id = PREFIX + ACTIONS[i].shortId;
			this._stepList.addStep(ACTIONS[i]);
			// all action ids get routed to the 'handleAction' method
			ActionRegistry.registerAction(id, this, "handleAction");
		}
		this._wizardStepArea.appendChild(this._stepList.domNode);
	}, 
	
	_defaultAction: function() {
		// by default, show the first section from the left-nav
		this.handleAction({action: PREFIX + ACTIONS[0].shortId});
	},
	
	handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("WizardPage.js#handleAction: missing required param 'args.action'");
			return;
		}
		// parse out the short id from the fully-qualified id
		var longId = args.action;
		var shortId = longId.substring(longId.lastIndexOf(".") + 1);
		
		// set the title for the action
		var actionEntry = ACTION_MAP[shortId];
		getWorkbench().setTitle(actionEntry.label);
		
		// activate action if not already
		if (!actionEntry.active) {
			this._stepList.activateStep(actionEntry, this);
			actionEntry.active = true;
		}
		
		// highlight the selected link and unhighlight the others
		var allLinks = this._stepSection.getElementsByTagName("A");
		for( var i = 0; i < allLinks.length; i++) {
			if (dojo.hasClass(allLinks[i], "selected") && allLinks[i].__jazzActionId !== shortId) {
				//var startCoords = dojo.coords(allLinks[i],true);
				allLinks[i].className = "";
			} else {
				if (allLinks[i].__jazzActionId === shortId) {
					allLinks[i].className = "selected";
					//var endCoords = dojo.coords(allLinks[i], true);
				} else {
					allLinks[i].className = "";
				}	
			}	
		}
		this[shortId]();
	},
	
	_createNavigationForAction: function(shortId) {
		var actionEntry = ACTION_MAP[shortId];
		actionEntry.prefix = PREFIX;
		var step = getStepForAction(shortId);
		if (step > 0) {
			actionEntry.previousAction = ACTIONS[step-1].shortId;
		}
		
		if (step != (ACTIONS.length - 1)) {
			actionEntry.nextAction = ACTIONS[(step+1)].shortId;
		} 
		
		return new com.ibm.team.repository.web.ui.internal.wizard.WizardNavigation(actionEntry);
	},
	
	_updateWizardView: function(panel) {
		dojox.data.dom.removeChildren(this._wizardPanelArea);
		this._wizardPanelArea.appendChild(panel.domNode);
			
		this._wizardNavigationArea.appendChild(panel._navigation.domNode);
		if (this._currentFooter){
			this._currentFooter.parentNode.removeChild(this._currentFooter);
		};
		this._currentFooter = panel._navigation.domNode;
	},
	
	showIntro: function() {
		if (!this._intro) {	
			var introNavigation = this._createNavigationForAction("showIntro");		
			var navArgs = {
				customAction: "#action=" + PREFIX + ACTIONS[1].shortId,
				fastAction: "#action=" + PREFIX + SETUP_USERS,
				wizardNav: introNavigation
			}
			this._intro = new com.ibm.team.repository.web.ui.internal.wizard.WizardIntro(navArgs);
			dojo.connect(this._intro, "onCustomButtonClick", this, function() {
				this._updateFastPath(false);
			});
			dojo.connect(this._intro, "onFastButtonClick", this, function() {
				this._updateFastPath(true);
			});
		}
		this._updateWizardView(this._intro);
 	}, 

	_updateFastPath: function (isFast){
		this._customPath = !isFast;
		if (!this._users) {
			this._createSetupUsersPanel();
		}
		if (isFast) {
			this._users._navigation.previousAction = "showIntro";
		} else {			
			this._users._navigation.previousAction = ACTIONS[ACTIONS.length-3].shortId;
		}
	},
	
	_createSetupUsersPanel: function () {
		var userNavigation = this._createNavigationForAction(SETUP_USERS);
		this._users = new com.ibm.team.repository.web.ui.internal.wizard.SetupUsers({wizardNav: userNavigation});
	},
	
	configureDatabase: function(args) {
		if (!this._customPath){
			this._updateFastPath(false)
		}
		
		if (!this._database) {
			var databaseNavigation = this._createNavigationForAction("configureDatabase");
			this._database = new com.ibm.team.repository.web.ui.internal.wizard.ConfigureDatabase({wizardNav: databaseNavigation});
		}
		this._updateWizardView(this._database);	
	},
	
	enableEmail: function(args) {
		if (!this._email) {
			var emailNavigation = this._createNavigationForAction("enableEmail");
			this._email = new com.ibm.team.repository.web.ui.internal.wizard.EnableEmail({wizardNav: emailNavigation});
		}
		this._updateWizardView(this._email);	
	},
	
	setupUsers: function() {
		if (!this._users) {
			this._createSetupUsersPanel();
		} else {
			this._users.refreshAvailabilityState();
		}
		this._updateWizardView(this._users);	
	},

	showReview: function() {
		if (!this._review) {
			var reviewNavigation = this._createNavigationForAction("showReview");
			this._review = new com.ibm.team.repository.web.ui.internal.wizard.WizardReview({wizardNav: reviewNavigation});
		} 
		this._review._refresh();
		this._updateWizardView(this._review);
 	}, 
	
	_showUnauthorized: function(){
		if (!this._unauth) {
			var noNavigation = new com.ibm.team.repository.web.ui.internal.wizard.WizardNavigation()
			this._unauth = new com.ibm.team.repository.web.ui.internal.wizard.NoAuth({wizardNav: noNavigation});
		}
		this._updateWizardView(this._unauth);	
	}
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.NoAuth", [dijit._Widget, dijit._Templated], {

	templateString: "<div><h2>Unauthorized Login</h2><p>You do not have the required role to access the Jazz Team Server Setup.</p></div>",

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.NoAuth.superclass.postCreate.apply(this);
		this._navigation = this.wizardNav; 
		this._navigation._wizardPreviousButton.className = "noDisplay";
		this._navigation._wizardNextButton.className = "noDisplay";
		this._navigation._wizardFinishButton.className = "noDisplay";
	}
});


var WizardPage = com.ibm.team.repository.web.ui.internal.wizard.WizardPage;

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

})();
